package fr.inria.phoenix.diasuite.framework.context.alert;

import fr.inria.diagen.core.ServiceConfiguration;
import fr.inria.diagen.core.exception.InvocationException;
import fr.inria.diagen.core.service.composite.CompositeException;
import fr.inria.diagen.core.network.RemoteServiceInfo;
import fr.inria.diagen.core.service.local.Service;
import fr.inria.diagen.core.service.proxy.Proxy;

import fr.inria.phoenix.diasuite.framework.device.pallettracker.StateFromPalletTracker;
import fr.inria.phoenix.diasuite.framework.device.timer.TimerTriggeredFromTimer;

/**
 * <pre>
 * context Alert as String {
 * 	when provided timerTriggered from Timer 
 * 	maybe publish;
 * 	when provided state from PalletTracker
 * 	no publish;
 * }
 * </pre>
 */
@SuppressWarnings("all")
public abstract class AbstractAlert extends Service {
    
    public AbstractAlert(ServiceConfiguration serviceConfiguration) {
        super("/Context/Alert/", serviceConfiguration);
    }
    
    // Methods from the Service class
    @Override
    protected final void internalPostInitialize() {
        postInitialize();
    }
    
    @Override
    protected void postInitialize() {
        // Default implementation of post initialize: subscribe to all required devices
        discoverPalletTrackerForSubscribe.all().subscribeState(); // subscribe to state from all PalletTracker devices
        discoverTimerForSubscribe.all().subscribeTimerTriggered(); // subscribe to timerTriggered from all Timer devices
    }
    
    @Override
    public final void valueReceived(java.util.Map<String, Object> properties, RemoteServiceInfo source, String eventName, Object value, Object... indexes) {
        if (eventName.equals("state") && source.isCompatible("/Device/Device/PhysicalDevice/Sensor/PalletTracker/")) {
            StateFromPalletTracker stateFromPalletTracker = new StateFromPalletTracker(this, source, (fr.inria.phoenix.diasuite.framework.datatype.state.State) value);
            
            onStateFromPalletTracker(stateFromPalletTracker);
        }
        if (eventName.equals("timerTriggered") && source.isCompatible("/Device/Device/Service/BuiltIn/Timer/")) {
            TimerTriggeredFromTimer timerTriggeredFromTimer = new TimerTriggeredFromTimer(this, source, (java.lang.String) value,
                    (java.lang.String) indexes[0]);
            
            AlertValuePublishable returnValue = onTimerTriggeredFromTimer(timerTriggeredFromTimer);
            if(returnValue != null && returnValue.doPublish()) {
                setAlert(returnValue.getValue());
            }
        }
    }
    
    @Override
    public final Object getValueCalled(java.util.Map<String, Object> properties, RemoteServiceInfo source, String valueName,
            Object... indexes) throws Exception {
        if (valueName.equals("alert")) {
            return getLastValue();
        }
        throw new InvocationException("Unsupported method call: " + valueName);
    }
    // End of methods from the Service class
    
    // Code relative to the return value of the context
    private java.lang.String contextValue;
    
    private void setAlert(java.lang.String newContextValue) {
        contextValue = newContextValue;
        getProcessor().publishValue(getOutProperties(), "alert", newContextValue);
    }
    
    /**
     * Get the last value of the context
     * 
     * @return the latest value published by the context
     */
    protected final java.lang.String getLastValue() {
        return contextValue;
    }
    
    /**
     * A class that represents a value that might be published for the <code>Alert</code> context. It is used by
     * event methods that might or might not publish values for this context.
     */
    protected final static class AlertValuePublishable {
        
        // The value of the context
        private java.lang.String value;
        // Whether the value should be published or not
        private boolean doPublish;
        
        public AlertValuePublishable(java.lang.String value, boolean doPublish) {
            this.value = value;
            this.doPublish = doPublish;
        }
        
        /**
         * @return the value of the context that might be published
         */
        public java.lang.String getValue() {
            return value;
        }
        
        /**
         * Sets the value that might be published
         * 
         * @param value the value that will be published if {@link #doPublish()} returns true
         */
        public void setValue(java.lang.String value) {
            this.value = value;
        }
        
        /**
         * @return true if the value should be published
         */
        public boolean doPublish() {
            return doPublish;
        }
        
        /**
         * Set the value to be publishable or not
         * 
         * @param doPublish if true, the value will be published
         */
        public void setDoPublish(boolean doPublish) {
            this.doPublish = doPublish;
        }
    }
    // End of code relative to the return value of the context
    
    // Interaction contract implementation
    /**
     * This method is called when a <code>Timer</code> device on which we have subscribed publish on its <code>timerTriggered</code> source.
     * 
     * <pre>
     * when provided timerTriggered from Timer 
     * 	maybe publish;
     * </pre>
     * 
     * @param timerTriggeredFromTimer the value of the <code>timerTriggered</code> source and the <code>Timer</code> device that published the value.
     * @return a {@link AlertValuePublishable} that says if the context should publish a value and which value it should publish
     */
    protected abstract AlertValuePublishable onTimerTriggeredFromTimer(TimerTriggeredFromTimer timerTriggeredFromTimer);
    
    /**
     * This method is called when a <code>PalletTracker</code> device on which we have subscribed publish on its <code>state</code> source.
     * 
     * <pre>
     * when provided state from PalletTracker
     * 	no publish;
     * </pre>
     * 
     * @param stateFromPalletTracker the value of the <code>state</code> source and the <code>PalletTracker</code> device that published the value.
     */
    protected abstract void onStateFromPalletTracker(StateFromPalletTracker stateFromPalletTracker);
    
    // End of interaction contract implementation
    
    // Discover part for Timer devices
    /**
     * Use this object to discover Timer devices.
     * 
     * <pre>
     * device Timer extends BuiltIn {
     * 	source timerTriggered as String indexed by timerId as String;
     * 	action ScheduleTimer;
     * }
     * </pre>
     * 
     * @see TimerDiscoverer
     */
    protected final TimerDiscoverer discoverTimerForSubscribe = new TimerDiscoverer(this);
    
    /**
     * Discover object that will exposes the <code>Timer</code> devices that can be discovered
     * 
     * <pre>
     * device Timer extends BuiltIn {
     * 	source timerTriggered as String indexed by timerId as String;
     * 	action ScheduleTimer;
     * }
     * </pre>
     */
    protected final static class TimerDiscoverer {
        private Service serviceParent;
        
        private TimerDiscoverer(Service serviceParent) {
            super();
            this.serviceParent = serviceParent;
        }
        
        private TimerComposite instantiateComposite() {
            return new TimerComposite(serviceParent);
        }
        
        /**
         * Returns a composite of all accessible <code>Timer</code> devices
         * 
         * @return a {@link TimerComposite} object composed of all discoverable <code>Timer</code>
         */
        public TimerComposite all() {
            return instantiateComposite();
        }
        
        /**
         * Returns a proxy to one out of all accessible <code>Timer</code> devices
         * 
         * @return a {@link TimerProxy} object pointing to a random discoverable <code>Timer</code> device
         */
        public TimerProxy anyOne() {
            return all().anyOne();
        }
        
        /**
         * Returns a composite of all accessible <code>Timer</code> devices whose attribute <code>id</code> matches a given value.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return a {@link TimerComposite} object composed of all matching <code>Timer</code> devices
         */
        public TimerComposite whereId(java.lang.String id) throws CompositeException {
            return instantiateComposite().andId(id);
        }
    }
    
    /**
     * A composite of several <code>Timer</code> devices
     * 
     * <pre>
     * device Timer extends BuiltIn {
     * 	source timerTriggered as String indexed by timerId as String;
     * 	action ScheduleTimer;
     * }
     * </pre>
     */
    protected final static class TimerComposite extends fr.inria.diagen.core.service.composite.Composite<TimerProxy> {
        private TimerComposite(Service serviceParent) {
            super(serviceParent, "/Device/Device/Service/BuiltIn/Timer/");
        }
        
        @Override
        protected TimerProxy instantiateProxy(RemoteServiceInfo rsi) {
            return new TimerProxy(service, rsi);
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>id</code>.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return this {@link TimerComposite}, filtered using the attribute <code>id</code>.
         */
        public TimerComposite andId(java.lang.String id) throws CompositeException {
            filterByAttribute("id", id);
            return this;
        }
        
        /**
         * Subscribes to the <code>timerTriggered</code> source. After a call to this method, the context will be notified when a
         * <code>Timer</code> device of this composite publishes a value on its <code>timerTriggered</code> source.
         */
        public void subscribeTimerTriggered() {
            subscribeValue("timerTriggered");
        }
        
        /**
         * Unsubscribes from the <code>timerTriggered</code> source. After a call to this method, the context will no more be notified
         * when a <code>Timer</code> device of this composite publishes a value on its <code>timerTriggered</code> source.
         */
        public void unsubscribeTimerTriggered() {
            unsubscribeValue("timerTriggered");
        }
    }
    
    /**
     * A proxy to one <code>Timer</code> device
     * 
     * <pre>
     * device Timer extends BuiltIn {
     * 	source timerTriggered as String indexed by timerId as String;
     * 	action ScheduleTimer;
     * }
     * </pre>
     */
    protected final static class TimerProxy extends Proxy {
        private TimerProxy(Service service, RemoteServiceInfo remoteServiceInfo) {
            super(service, remoteServiceInfo);
        }
        
        /**
         * Subscribes to the <code>timerTriggered</code> source. After a call to this method, the context will be notified when the
         * <code>Timer</code> device of this proxy publishes a value on its <code>timerTriggered</code> source.
         */
        public void subscribeTimerTriggered() {
            subscribeValue("timerTriggered");
        }
        
        /**
         * Unsubscribes from the <code>timerTriggered</code> source. After a call to this method, the context will no more be notified
         * when the <code>Timer</code> device of this proxy publishes a value on its <code>timerTriggered</code> source.
         */
        public void unsubscribeTimerTriggered() {
            unsubscribeValue("timerTriggered");
        }
        
        /**
         * @return the value of the <code>id</code> attribute
         */
        public java.lang.String id() {
            return (java.lang.String) callGetValue("id");
        }
    }
    // End of discover part for Timer devices
    
    // Discover part for PalletTracker devices
    /**
     * Use this object to discover PalletTracker devices.
     * 
     * <pre>
     * device PalletTracker extends Sensor {
     * 	source state as State;
     * 	source position as GPSCoordinates indexed by params as String [];
     * }
     * </pre>
     * 
     * @see PalletTrackerDiscoverer
     */
    protected final PalletTrackerDiscoverer discoverPalletTrackerForSubscribe = new PalletTrackerDiscoverer(this);
    
    /**
     * Discover object that will exposes the <code>PalletTracker</code> devices that can be discovered
     * 
     * <pre>
     * device PalletTracker extends Sensor {
     * 	source state as State;
     * 	source position as GPSCoordinates indexed by params as String [];
     * }
     * </pre>
     */
    protected final static class PalletTrackerDiscoverer {
        private Service serviceParent;
        
        private PalletTrackerDiscoverer(Service serviceParent) {
            super();
            this.serviceParent = serviceParent;
        }
        
        private PalletTrackerComposite instantiateComposite() {
            return new PalletTrackerComposite(serviceParent);
        }
        
        /**
         * Returns a composite of all accessible <code>PalletTracker</code> devices
         * 
         * @return a {@link PalletTrackerComposite} object composed of all discoverable <code>PalletTracker</code>
         */
        public PalletTrackerComposite all() {
            return instantiateComposite();
        }
        
        /**
         * Returns a proxy to one out of all accessible <code>PalletTracker</code> devices
         * 
         * @return a {@link PalletTrackerProxy} object pointing to a random discoverable <code>PalletTracker</code> device
         */
        public PalletTrackerProxy anyOne() {
            return all().anyOne();
        }
        
        /**
         * Returns a composite of all accessible <code>PalletTracker</code> devices whose attribute <code>id</code> matches a given value.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return a {@link PalletTrackerComposite} object composed of all matching <code>PalletTracker</code> devices
         */
        public PalletTrackerComposite whereId(java.lang.String id) throws CompositeException {
            return instantiateComposite().andId(id);
        }
        
        /**
         * Returns a composite of all accessible <code>PalletTracker</code> devices whose attribute <code>location</code> matches a given value.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return a {@link PalletTrackerComposite} object composed of all matching <code>PalletTracker</code> devices
         */
        public PalletTrackerComposite whereLocation(java.lang.String location) throws CompositeException {
            return instantiateComposite().andLocation(location);
        }
        
        /**
         * Returns a composite of all accessible <code>PalletTracker</code> devices whose attribute <code>user</code> matches a given value.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return a {@link PalletTrackerComposite} object composed of all matching <code>PalletTracker</code> devices
         */
        public PalletTrackerComposite whereUser(java.lang.String user) throws CompositeException {
            return instantiateComposite().andUser(user);
        }
    }
    
    /**
     * A composite of several <code>PalletTracker</code> devices
     * 
     * <pre>
     * device PalletTracker extends Sensor {
     * 	source state as State;
     * 	source position as GPSCoordinates indexed by params as String [];
     * }
     * </pre>
     */
    protected final static class PalletTrackerComposite extends fr.inria.diagen.core.service.composite.Composite<PalletTrackerProxy> {
        private PalletTrackerComposite(Service serviceParent) {
            super(serviceParent, "/Device/Device/PhysicalDevice/Sensor/PalletTracker/");
        }
        
        @Override
        protected PalletTrackerProxy instantiateProxy(RemoteServiceInfo rsi) {
            return new PalletTrackerProxy(service, rsi);
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>id</code>.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return this {@link PalletTrackerComposite}, filtered using the attribute <code>id</code>.
         */
        public PalletTrackerComposite andId(java.lang.String id) throws CompositeException {
            filterByAttribute("id", id);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>location</code>.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return this {@link PalletTrackerComposite}, filtered using the attribute <code>location</code>.
         */
        public PalletTrackerComposite andLocation(java.lang.String location) throws CompositeException {
            filterByAttribute("location", location);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>user</code>.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return this {@link PalletTrackerComposite}, filtered using the attribute <code>user</code>.
         */
        public PalletTrackerComposite andUser(java.lang.String user) throws CompositeException {
            filterByAttribute("user", user);
            return this;
        }
        
        /**
         * Subscribes to the <code>state</code> source. After a call to this method, the context will be notified when a
         * <code>PalletTracker</code> device of this composite publishes a value on its <code>state</code> source.
         */
        public void subscribeState() {
            subscribeValue("state");
        }
        
        /**
         * Unsubscribes from the <code>state</code> source. After a call to this method, the context will no more be notified
         * when a <code>PalletTracker</code> device of this composite publishes a value on its <code>state</code> source.
         */
        public void unsubscribeState() {
            unsubscribeValue("state");
        }
    }
    
    /**
     * A proxy to one <code>PalletTracker</code> device
     * 
     * <pre>
     * device PalletTracker extends Sensor {
     * 	source state as State;
     * 	source position as GPSCoordinates indexed by params as String [];
     * }
     * </pre>
     */
    protected final static class PalletTrackerProxy extends Proxy {
        private PalletTrackerProxy(Service service, RemoteServiceInfo remoteServiceInfo) {
            super(service, remoteServiceInfo);
        }
        
        /**
         * Subscribes to the <code>state</code> source. After a call to this method, the context will be notified when the
         * <code>PalletTracker</code> device of this proxy publishes a value on its <code>state</code> source.
         */
        public void subscribeState() {
            subscribeValue("state");
        }
        
        /**
         * Unsubscribes from the <code>state</code> source. After a call to this method, the context will no more be notified
         * when the <code>PalletTracker</code> device of this proxy publishes a value on its <code>state</code> source.
         */
        public void unsubscribeState() {
            unsubscribeValue("state");
        }
        
        /**
         * @return the value of the <code>id</code> attribute
         */
        public java.lang.String id() {
            return (java.lang.String) callGetValue("id");
        }
        
        /**
         * @return the value of the <code>location</code> attribute
         */
        public java.lang.String location() {
            return (java.lang.String) callGetValue("location");
        }
        
        /**
         * @return the value of the <code>user</code> attribute
         */
        public java.lang.String user() {
            return (java.lang.String) callGetValue("user");
        }
    }
    // End of discover part for PalletTracker devices
}
