package fr.inria.phoenix.diasuite.framework.context.event;

import fr.inria.diagen.core.ServiceConfiguration;
import fr.inria.diagen.core.exception.InvocationException;
import fr.inria.diagen.core.service.composite.CompositeException;
import fr.inria.diagen.core.network.RemoteServiceInfo;
import fr.inria.diagen.core.service.local.Service;
import fr.inria.diagen.core.service.proxy.Proxy;

import fr.inria.phoenix.diasuite.framework.device.input.MessageFromInput;

/**
 * <pre>
 * context Event as String indexed by periodicity as Integer {
 * 	when provided message from Input
 * 	maybe publish;	
 * }
 * </pre>
 */
@SuppressWarnings("all")
public abstract class AbstractEvent extends Service {
    
    public AbstractEvent(ServiceConfiguration serviceConfiguration) {
        super("/Context/Event/", serviceConfiguration);
    }
    
    // Methods from the Service class
    @Override
    protected final void internalPostInitialize() {
        postInitialize();
    }
    
    @Override
    protected void postInitialize() {
        // Default implementation of post initialize: subscribe to all required devices
        discoverInputForSubscribe.all().subscribeMessage(); // subscribe to message from all Input devices
    }
    
    @Override
    public final void valueReceived(java.util.Map<String, Object> properties, RemoteServiceInfo source, String eventName, Object value, Object... indexes) {
        if (eventName.equals("message") && source.isCompatible("/Device/Device/Service/Input/")) {
            MessageFromInput messageFromInput = new MessageFromInput(this, source, (java.lang.String) value,
                    (java.util.List<java.lang.String>) indexes[0]);
            
            EventValuePublishable returnValue = onMessageFromInput(messageFromInput);
            if(returnValue != null && returnValue.doPublish()) {
                setEvent(returnValue.getValue(), returnValue.getPeriodicity());
            }
        }
    }
    
    @Override
    public final Object getValueCalled(java.util.Map<String, Object> properties, RemoteServiceInfo source, String valueName,
            Object... indexes) throws Exception {
        if (valueName.equals("event")) {
            return getLastValue((java.lang.Integer) indexes[0]);
        }
        throw new InvocationException("Unsupported method call: " + valueName);
    }
    // End of methods from the Service class
    
    // Code relative to the return value of the context
    private java.util.Map<EventIndices, java.lang.String> contextValues = new java.util.HashMap<EventIndices, java.lang.String>();
    
    private void setEvent(java.lang.String newContextValue, java.lang.Integer periodicity) {
        EventIndices indices = new EventIndices(periodicity);
        contextValues.put(indices, newContextValue);
        getProcessor().publishValue(getOutProperties(), "event", newContextValue, periodicity);
    }
    
    /**
     * Get the last value of the context
     * 
     * @param periodicity the index <code>periodicity</code> for the last value
     * @return the latest value published by the context for the given indexes
     */
    protected final java.lang.String getLastValue(java.lang.Integer periodicity) {
        EventIndices indices = new EventIndices(periodicity);
        return contextValues.get(indices);
    }
    
    /**
     * A class that represents a value that might be published for the <code>Event</code> context. It is used by
     * event methods that might or might not publish values for this context.
     */
    protected final static class EventValuePublishable {
        
        // The value of the context
        private java.lang.String value;
        // The periodicity index
        private java.lang.Integer periodicity;
        // Whether the value should be published or not
        private boolean doPublish;
        
        public EventValuePublishable(java.lang.String value,
                java.lang.Integer periodicity,
                boolean doPublish) {
            this.value = value;
            this.periodicity = periodicity;
            this.doPublish = doPublish;
        }
        
        /**
         * @return the value of the context that might be published
         */
        public java.lang.String getValue() {
            return value;
        }
        
        /**
         * Sets the value that might be published
         * 
         * @param value the value that will be published if {@link #doPublish()} returns true
         */
        public void setValue(java.lang.String value) {
            this.value = value;
        }
        
        /**
         * @return the <code>periodicity</code> index for the value of the context that might be published
         */
        public java.lang.Integer getPeriodicity() {
            return periodicity;
        }
        
        /**
         * Sets the <code>periodicity</code> index for the value that might be published
         * 
         * @param periodicity the <code>periodicity</code> index
         */
        public void setPeriodicity(java.lang.Integer periodicity) {
            this.periodicity = periodicity;
        }
        
        /**
         * @return true if the value should be published
         */
        public boolean doPublish() {
            return doPublish;
        }
        
        /**
         * Set the value to be publishable or not
         * 
         * @param doPublish if true, the value will be published
         */
        public void setDoPublish(boolean doPublish) {
            this.doPublish = doPublish;
        }
    }
    // End of code relative to the return value of the context
    
    // Interaction contract implementation
    /**
     * This method is called when a <code>Input</code> device on which we have subscribed publish on its <code>message</code> source.
     * 
     * <pre>
     * when provided message from Input
     * 	maybe publish;
     * </pre>
     * 
     * @param messageFromInput the value of the <code>message</code> source and the <code>Input</code> device that published the value.
     * @return a {@link EventValuePublishable} that says if the context should publish a value and which value it should publish
     */
    protected abstract EventValuePublishable onMessageFromInput(MessageFromInput messageFromInput);
    
    // End of interaction contract implementation
    
    // Discover part for Input devices
    /**
     * Use this object to discover Input devices.
     * 
     * <pre>
     * device Input extends Service {
     * 	source message as String indexed by params as String [];
     * }
     * </pre>
     * 
     * @see InputDiscoverer
     */
    protected final InputDiscoverer discoverInputForSubscribe = new InputDiscoverer(this);
    
    /**
     * Discover object that will exposes the <code>Input</code> devices that can be discovered
     * 
     * <pre>
     * device Input extends Service {
     * 	source message as String indexed by params as String [];
     * }
     * </pre>
     */
    protected final static class InputDiscoverer {
        private Service serviceParent;
        
        private InputDiscoverer(Service serviceParent) {
            super();
            this.serviceParent = serviceParent;
        }
        
        private InputComposite instantiateComposite() {
            return new InputComposite(serviceParent);
        }
        
        /**
         * Returns a composite of all accessible <code>Input</code> devices
         * 
         * @return a {@link InputComposite} object composed of all discoverable <code>Input</code>
         */
        public InputComposite all() {
            return instantiateComposite();
        }
        
        /**
         * Returns a proxy to one out of all accessible <code>Input</code> devices
         * 
         * @return a {@link InputProxy} object pointing to a random discoverable <code>Input</code> device
         */
        public InputProxy anyOne() {
            return all().anyOne();
        }
        
        /**
         * Returns a composite of all accessible <code>Input</code> devices whose attribute <code>id</code> matches a given value.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return a {@link InputComposite} object composed of all matching <code>Input</code> devices
         */
        public InputComposite whereId(java.lang.String id) throws CompositeException {
            return instantiateComposite().andId(id);
        }
    }
    
    /**
     * A composite of several <code>Input</code> devices
     * 
     * <pre>
     * device Input extends Service {
     * 	source message as String indexed by params as String [];
     * }
     * </pre>
     */
    protected final static class InputComposite extends fr.inria.diagen.core.service.composite.Composite<InputProxy> {
        private InputComposite(Service serviceParent) {
            super(serviceParent, "/Device/Device/Service/Input/");
        }
        
        @Override
        protected InputProxy instantiateProxy(RemoteServiceInfo rsi) {
            return new InputProxy(service, rsi);
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>id</code>.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return this {@link InputComposite}, filtered using the attribute <code>id</code>.
         */
        public InputComposite andId(java.lang.String id) throws CompositeException {
            filterByAttribute("id", id);
            return this;
        }
        
        /**
         * Subscribes to the <code>message</code> source. After a call to this method, the context will be notified when a
         * <code>Input</code> device of this composite publishes a value on its <code>message</code> source.
         */
        public void subscribeMessage() {
            subscribeValue("message");
        }
        
        /**
         * Unsubscribes from the <code>message</code> source. After a call to this method, the context will no more be notified
         * when a <code>Input</code> device of this composite publishes a value on its <code>message</code> source.
         */
        public void unsubscribeMessage() {
            unsubscribeValue("message");
        }
    }
    
    /**
     * A proxy to one <code>Input</code> device
     * 
     * <pre>
     * device Input extends Service {
     * 	source message as String indexed by params as String [];
     * }
     * </pre>
     */
    protected final static class InputProxy extends Proxy {
        private InputProxy(Service service, RemoteServiceInfo remoteServiceInfo) {
            super(service, remoteServiceInfo);
        }
        
        /**
         * Subscribes to the <code>message</code> source. After a call to this method, the context will be notified when the
         * <code>Input</code> device of this proxy publishes a value on its <code>message</code> source.
         */
        public void subscribeMessage() {
            subscribeValue("message");
        }
        
        /**
         * Unsubscribes from the <code>message</code> source. After a call to this method, the context will no more be notified
         * when the <code>Input</code> device of this proxy publishes a value on its <code>message</code> source.
         */
        public void unsubscribeMessage() {
            unsubscribeValue("message");
        }
        
        /**
         * @return the value of the <code>id</code> attribute
         */
        public java.lang.String id() {
            return (java.lang.String) callGetValue("id");
        }
    }
    // End of discover part for Input devices
}
