package fr.inria.phoenix.diasuite.framework.context.state;

import fr.inria.diagen.core.ServiceConfiguration;
import fr.inria.diagen.core.exception.InvocationException;
import fr.inria.diagen.core.service.composite.CompositeException;
import fr.inria.diagen.core.network.RemoteServiceInfo;
import fr.inria.diagen.core.service.local.Service;
import fr.inria.diagen.core.service.proxy.Proxy;

import fr.inria.phoenix.diasuite.framework.device.pallettracker.StateFromPalletTracker;

/**
 * context Warehouse as String indexed by position as GPSCoordinates {
 * when required get warehouse from PalletTrackingDB;
 * }
 * 
 * <pre>
 * context State as PalletState {
 * 	when provided state from PalletTracker 
 * 	get Position, Temperature	 
 * 	always publish;
 * }
 * </pre>
 */
@SuppressWarnings("all")
public abstract class AbstractState extends Service {
    
    public AbstractState(ServiceConfiguration serviceConfiguration) {
        super("/Context/State/", serviceConfiguration);
    }
    
    // Methods from the Service class
    @Override
    protected final void internalPostInitialize() {
        postInitialize();
    }
    
    @Override
    protected void postInitialize() {
        // Default implementation of post initialize: subscribe to all required devices
        discoverPalletTrackerForSubscribe.all().subscribeState(); // subscribe to state from all PalletTracker devices
    }
    
    @Override
    public final void valueReceived(java.util.Map<String, Object> properties, RemoteServiceInfo source, String eventName, Object value, Object... indexes) {
        if (eventName.equals("state") && source.isCompatible("/Device/Device/PhysicalDevice/Sensor/PalletTracker/")) {
            StateFromPalletTracker stateFromPalletTracker = new StateFromPalletTracker(this, source, (fr.inria.phoenix.diasuite.framework.datatype.state.State) value);
            
            setState(onStateFromPalletTracker(stateFromPalletTracker, new DiscoverForStateFromPalletTracker()));
        }
    }
    
    @Override
    public final Object getValueCalled(java.util.Map<String, Object> properties, RemoteServiceInfo source, String valueName,
            Object... indexes) throws Exception {
        if (valueName.equals("state")) {
            return getLastValue();
        }
        throw new InvocationException("Unsupported method call: " + valueName);
    }
    // End of methods from the Service class
    
    // Code relative to the return value of the context
    private fr.inria.phoenix.diasuite.framework.datatype.palletstate.PalletState contextValue;
    
    private void setState(fr.inria.phoenix.diasuite.framework.datatype.palletstate.PalletState newContextValue) {
        contextValue = newContextValue;
        getProcessor().publishValue(getOutProperties(), "state", newContextValue);
    }
    
    /**
     * Get the last value of the context
     * 
     * @return the latest value published by the context
     */
    protected final fr.inria.phoenix.diasuite.framework.datatype.palletstate.PalletState getLastValue() {
        return contextValue;
    }
    // End of code relative to the return value of the context
    
    // Interaction contract implementation
    /**
     * This method is called when a <code>PalletTracker</code> device on which we have subscribed publish on its <code>state</code> source.
     * 
     * <pre>
     * when provided state from PalletTracker 
     * 	get Position, Temperature	 
     * 	always publish;
     * </pre>
     * 
     * @param stateFromPalletTracker the value of the <code>state</code> source and the <code>PalletTracker</code> device that published the value.
     * @param discover a discover object to get value from devices and contexts
     * @return the value to publish
     */
    protected abstract fr.inria.phoenix.diasuite.framework.datatype.palletstate.PalletState onStateFromPalletTracker(StateFromPalletTracker stateFromPalletTracker, DiscoverForStateFromPalletTracker discover);
    
    // End of interaction contract implementation
    
    // Discover part for PalletTracker devices
    /**
     * Use this object to discover PalletTracker devices.
     * 
     * <pre>
     * device PalletTracker extends Sensor {
     * 	source state as State;
     * 	source position as GPSCoordinates indexed by params as String [];
     * }
     * </pre>
     * 
     * @see PalletTrackerDiscoverer
     */
    protected final PalletTrackerDiscoverer discoverPalletTrackerForSubscribe = new PalletTrackerDiscoverer(this);
    
    /**
     * Discover object that will exposes the <code>PalletTracker</code> devices that can be discovered
     * 
     * <pre>
     * device PalletTracker extends Sensor {
     * 	source state as State;
     * 	source position as GPSCoordinates indexed by params as String [];
     * }
     * </pre>
     */
    protected final static class PalletTrackerDiscoverer {
        private Service serviceParent;
        
        private PalletTrackerDiscoverer(Service serviceParent) {
            super();
            this.serviceParent = serviceParent;
        }
        
        private PalletTrackerComposite instantiateComposite() {
            return new PalletTrackerComposite(serviceParent);
        }
        
        /**
         * Returns a composite of all accessible <code>PalletTracker</code> devices
         * 
         * @return a {@link PalletTrackerComposite} object composed of all discoverable <code>PalletTracker</code>
         */
        public PalletTrackerComposite all() {
            return instantiateComposite();
        }
        
        /**
         * Returns a proxy to one out of all accessible <code>PalletTracker</code> devices
         * 
         * @return a {@link PalletTrackerProxy} object pointing to a random discoverable <code>PalletTracker</code> device
         */
        public PalletTrackerProxy anyOne() {
            return all().anyOne();
        }
        
        /**
         * Returns a composite of all accessible <code>PalletTracker</code> devices whose attribute <code>id</code> matches a given value.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return a {@link PalletTrackerComposite} object composed of all matching <code>PalletTracker</code> devices
         */
        public PalletTrackerComposite whereId(java.lang.String id) throws CompositeException {
            return instantiateComposite().andId(id);
        }
        
        /**
         * Returns a composite of all accessible <code>PalletTracker</code> devices whose attribute <code>location</code> matches a given value.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return a {@link PalletTrackerComposite} object composed of all matching <code>PalletTracker</code> devices
         */
        public PalletTrackerComposite whereLocation(java.lang.String location) throws CompositeException {
            return instantiateComposite().andLocation(location);
        }
        
        /**
         * Returns a composite of all accessible <code>PalletTracker</code> devices whose attribute <code>user</code> matches a given value.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return a {@link PalletTrackerComposite} object composed of all matching <code>PalletTracker</code> devices
         */
        public PalletTrackerComposite whereUser(java.lang.String user) throws CompositeException {
            return instantiateComposite().andUser(user);
        }
    }
    
    /**
     * A composite of several <code>PalletTracker</code> devices
     * 
     * <pre>
     * device PalletTracker extends Sensor {
     * 	source state as State;
     * 	source position as GPSCoordinates indexed by params as String [];
     * }
     * </pre>
     */
    protected final static class PalletTrackerComposite extends fr.inria.diagen.core.service.composite.Composite<PalletTrackerProxy> {
        private PalletTrackerComposite(Service serviceParent) {
            super(serviceParent, "/Device/Device/PhysicalDevice/Sensor/PalletTracker/");
        }
        
        @Override
        protected PalletTrackerProxy instantiateProxy(RemoteServiceInfo rsi) {
            return new PalletTrackerProxy(service, rsi);
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>id</code>.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return this {@link PalletTrackerComposite}, filtered using the attribute <code>id</code>.
         */
        public PalletTrackerComposite andId(java.lang.String id) throws CompositeException {
            filterByAttribute("id", id);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>location</code>.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return this {@link PalletTrackerComposite}, filtered using the attribute <code>location</code>.
         */
        public PalletTrackerComposite andLocation(java.lang.String location) throws CompositeException {
            filterByAttribute("location", location);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>user</code>.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return this {@link PalletTrackerComposite}, filtered using the attribute <code>user</code>.
         */
        public PalletTrackerComposite andUser(java.lang.String user) throws CompositeException {
            filterByAttribute("user", user);
            return this;
        }
        
        /**
         * Subscribes to the <code>state</code> source. After a call to this method, the context will be notified when a
         * <code>PalletTracker</code> device of this composite publishes a value on its <code>state</code> source.
         */
        public void subscribeState() {
            subscribeValue("state");
        }
        
        /**
         * Unsubscribes from the <code>state</code> source. After a call to this method, the context will no more be notified
         * when a <code>PalletTracker</code> device of this composite publishes a value on its <code>state</code> source.
         */
        public void unsubscribeState() {
            unsubscribeValue("state");
        }
    }
    
    /**
     * A proxy to one <code>PalletTracker</code> device
     * 
     * <pre>
     * device PalletTracker extends Sensor {
     * 	source state as State;
     * 	source position as GPSCoordinates indexed by params as String [];
     * }
     * </pre>
     */
    protected final static class PalletTrackerProxy extends Proxy {
        private PalletTrackerProxy(Service service, RemoteServiceInfo remoteServiceInfo) {
            super(service, remoteServiceInfo);
        }
        
        /**
         * Subscribes to the <code>state</code> source. After a call to this method, the context will be notified when the
         * <code>PalletTracker</code> device of this proxy publishes a value on its <code>state</code> source.
         */
        public void subscribeState() {
            subscribeValue("state");
        }
        
        /**
         * Unsubscribes from the <code>state</code> source. After a call to this method, the context will no more be notified
         * when the <code>PalletTracker</code> device of this proxy publishes a value on its <code>state</code> source.
         */
        public void unsubscribeState() {
            unsubscribeValue("state");
        }
        
        /**
         * @return the value of the <code>id</code> attribute
         */
        public java.lang.String id() {
            return (java.lang.String) callGetValue("id");
        }
        
        /**
         * @return the value of the <code>location</code> attribute
         */
        public java.lang.String location() {
            return (java.lang.String) callGetValue("location");
        }
        
        /**
         * @return the value of the <code>user</code> attribute
         */
        public java.lang.String user() {
            return (java.lang.String) callGetValue("user");
        }
    }
    // End of discover part for PalletTracker devices
    
    // Discover object for state from PalletTracker
    /**
     * An object to discover devices and contexts for the following interaction contract:
     * 
     * <code>
     * when provided state from PalletTracker 
     * 	get Position, Temperature	 
     * 	always publish;
     * </code>
     */
    protected final class DiscoverForStateFromPalletTracker {
        private final fr.inria.diagen.core.service.filter.service.ServiceFilter positionFilter = new fr.inria.diagen.core.service.filter.service.ServiceFilter("/Context/Position/");
        private final fr.inria.diagen.core.service.filter.service.ServiceFilter temperatureFilter = new fr.inria.diagen.core.service.filter.service.ServiceFilter("/Context/Temperature/");
        
        private RemoteServiceInfo getPositionRsi() {
            return getProcessor().discoverComponents(getOutProperties(), positionFilter).get(0);
        }
        
        private RemoteServiceInfo getTemperatureRsi() {
            return getProcessor().discoverComponents(getOutProperties(), temperatureFilter).get(0);
        }
        
        /**
         * Get the value of the <code>Position</code> context
         * 
         * @param id the value of the <code>id</code> index.
         * @return the value of the <code>Position</code> context
         */
        public fr.inria.phoenix.diasuite.framework.datatype.gpscoordinates.GPSCoordinates position(java.lang.String id) {
            return (fr.inria.phoenix.diasuite.framework.datatype.gpscoordinates.GPSCoordinates) getProcessor().callGetValue(getOutProperties(), getPositionRsi(), "position",
                    id);
        }
        
        /**
         * Get the value of the <code>Temperature</code> context
         * 
         * @param id the value of the <code>id</code> index.
         * @return the value of the <code>Temperature</code> context
         */
        public java.lang.Integer temperature(java.lang.String id) {
            return (java.lang.Integer) getProcessor().callGetValue(getOutProperties(), getTemperatureRsi(), "temperature",
                    id);
        }
    }
    // End of discover object for state from PalletTracker
}
