package fr.inria.phoenix.diasuite.framework.context.temperature;

import fr.inria.diagen.core.ServiceConfiguration;
import fr.inria.diagen.core.exception.InvocationException;
import fr.inria.diagen.core.service.composite.CompositeException;
import fr.inria.diagen.core.network.RemoteServiceInfo;
import fr.inria.diagen.core.service.local.Service;
import fr.inria.diagen.core.service.proxy.Proxy;

import fr.inria.phoenix.diasuite.framework.device.temperaturesensor.TemperatureFromTemperatureSensor;

/**
 * The provided temperature is indexed by id and published.
 * 
 * <pre>
 * context Temperature as Integer indexed by id as String {
 * 	when provided temperature from TemperatureSensor 
 * 	no publish;
 * 	when required;
 * }
 * </pre>
 */
@SuppressWarnings("all")
public abstract class AbstractTemperature extends Service {
    
    public AbstractTemperature(ServiceConfiguration serviceConfiguration) {
        super("/Context/Temperature/", serviceConfiguration);
    }
    
    // Methods from the Service class
    @Override
    protected final void internalPostInitialize() {
        postInitialize();
    }
    
    @Override
    protected void postInitialize() {
        // Default implementation of post initialize: subscribe to all required devices
        discoverTemperatureSensorForSubscribe.all().subscribeTemperature(); // subscribe to temperature from all TemperatureSensor devices
    }
    
    @Override
    public final void valueReceived(java.util.Map<String, Object> properties, RemoteServiceInfo source, String eventName, Object value, Object... indexes) {
        if (eventName.equals("temperature") && source.isCompatible("/Device/Device/PhysicalDevice/Sensor/TemperatureSensor/")) {
            TemperatureFromTemperatureSensor temperatureFromTemperatureSensor = new TemperatureFromTemperatureSensor(this, source, (fr.inria.phoenix.diasuite.framework.datatype.temperature.Temperature) value);
            
            onTemperatureFromTemperatureSensor(temperatureFromTemperatureSensor);
        }
    }
    
    @Override
    public final Object getValueCalled(java.util.Map<String, Object> properties, RemoteServiceInfo source, String valueName,
            Object... indexes) throws Exception {
        if (valueName.equals("temperature")) {
            return getValue((java.lang.String) indexes[0]);
        }
        throw new InvocationException("Unsupported method call: " + valueName);
    }
    // End of methods from the Service class
    
    // Code relative to the return value of the context
    private java.util.Map<TemperatureIndices, java.lang.Integer> contextValues = new java.util.HashMap<TemperatureIndices, java.lang.Integer>();
    
    private void setTemperature(java.lang.Integer newContextValue, java.lang.String id) {
        TemperatureIndices indices = new TemperatureIndices(id);
        contextValues.put(indices, newContextValue);
        getProcessor().publishValue(getOutProperties(), "temperature", newContextValue, id);
    }
    
    /**
     * Get the last value of the context
     * 
     * @param id the index <code>id</code> for the last value
     * @return the latest value published by the context for the given indexes
     */
    protected final java.lang.Integer getLastValue(java.lang.String id) {
        TemperatureIndices indices = new TemperatureIndices(id);
        return contextValues.get(indices);
    }
    // End of code relative to the return value of the context
    
    // Interaction contract implementation
    /**
     * This method is called when a <code>TemperatureSensor</code> device on which we have subscribed publish on its <code>temperature</code> source.
     * 
     * <pre>
     * when provided temperature from TemperatureSensor 
     * 	no publish;
     * </pre>
     * 
     * @param temperatureFromTemperatureSensor the value of the <code>temperature</code> source and the <code>TemperatureSensor</code> device that published the value.
     */
    protected abstract void onTemperatureFromTemperatureSensor(TemperatureFromTemperatureSensor temperatureFromTemperatureSensor);
    
    /**
     * This method is called when the value of this context is required.
     * 
     * <pre>
     * when required;
     * </pre>
     * 
     * @param id the value of the <code>id</code> index
     * @return the value of the context to return
     */
    protected abstract java.lang.Integer getValue(java.lang.String id);
    
    // End of interaction contract implementation
    
    // Discover part for TemperatureSensor devices
    /**
     * Use this object to discover TemperatureSensor devices.
     * 
     * <pre>
     * device TemperatureSensor extends Sensor {
     * 	source temperature as Temperature;
     * }
     * </pre>
     * 
     * @see TemperatureSensorDiscoverer
     */
    protected final TemperatureSensorDiscoverer discoverTemperatureSensorForSubscribe = new TemperatureSensorDiscoverer(this);
    
    /**
     * Discover object that will exposes the <code>TemperatureSensor</code> devices that can be discovered
     * 
     * <pre>
     * device TemperatureSensor extends Sensor {
     * 	source temperature as Temperature;
     * }
     * </pre>
     */
    protected final static class TemperatureSensorDiscoverer {
        private Service serviceParent;
        
        private TemperatureSensorDiscoverer(Service serviceParent) {
            super();
            this.serviceParent = serviceParent;
        }
        
        private TemperatureSensorComposite instantiateComposite() {
            return new TemperatureSensorComposite(serviceParent);
        }
        
        /**
         * Returns a composite of all accessible <code>TemperatureSensor</code> devices
         * 
         * @return a {@link TemperatureSensorComposite} object composed of all discoverable <code>TemperatureSensor</code>
         */
        public TemperatureSensorComposite all() {
            return instantiateComposite();
        }
        
        /**
         * Returns a proxy to one out of all accessible <code>TemperatureSensor</code> devices
         * 
         * @return a {@link TemperatureSensorProxy} object pointing to a random discoverable <code>TemperatureSensor</code> device
         */
        public TemperatureSensorProxy anyOne() {
            return all().anyOne();
        }
        
        /**
         * Returns a composite of all accessible <code>TemperatureSensor</code> devices whose attribute <code>id</code> matches a given value.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return a {@link TemperatureSensorComposite} object composed of all matching <code>TemperatureSensor</code> devices
         */
        public TemperatureSensorComposite whereId(java.lang.String id) throws CompositeException {
            return instantiateComposite().andId(id);
        }
        
        /**
         * Returns a composite of all accessible <code>TemperatureSensor</code> devices whose attribute <code>location</code> matches a given value.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return a {@link TemperatureSensorComposite} object composed of all matching <code>TemperatureSensor</code> devices
         */
        public TemperatureSensorComposite whereLocation(java.lang.String location) throws CompositeException {
            return instantiateComposite().andLocation(location);
        }
        
        /**
         * Returns a composite of all accessible <code>TemperatureSensor</code> devices whose attribute <code>user</code> matches a given value.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return a {@link TemperatureSensorComposite} object composed of all matching <code>TemperatureSensor</code> devices
         */
        public TemperatureSensorComposite whereUser(java.lang.String user) throws CompositeException {
            return instantiateComposite().andUser(user);
        }
    }
    
    /**
     * A composite of several <code>TemperatureSensor</code> devices
     * 
     * <pre>
     * device TemperatureSensor extends Sensor {
     * 	source temperature as Temperature;
     * }
     * </pre>
     */
    protected final static class TemperatureSensorComposite extends fr.inria.diagen.core.service.composite.Composite<TemperatureSensorProxy> {
        private TemperatureSensorComposite(Service serviceParent) {
            super(serviceParent, "/Device/Device/PhysicalDevice/Sensor/TemperatureSensor/");
        }
        
        @Override
        protected TemperatureSensorProxy instantiateProxy(RemoteServiceInfo rsi) {
            return new TemperatureSensorProxy(service, rsi);
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>id</code>.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return this {@link TemperatureSensorComposite}, filtered using the attribute <code>id</code>.
         */
        public TemperatureSensorComposite andId(java.lang.String id) throws CompositeException {
            filterByAttribute("id", id);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>location</code>.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return this {@link TemperatureSensorComposite}, filtered using the attribute <code>location</code>.
         */
        public TemperatureSensorComposite andLocation(java.lang.String location) throws CompositeException {
            filterByAttribute("location", location);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>user</code>.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return this {@link TemperatureSensorComposite}, filtered using the attribute <code>user</code>.
         */
        public TemperatureSensorComposite andUser(java.lang.String user) throws CompositeException {
            filterByAttribute("user", user);
            return this;
        }
        
        /**
         * Subscribes to the <code>temperature</code> source. After a call to this method, the context will be notified when a
         * <code>TemperatureSensor</code> device of this composite publishes a value on its <code>temperature</code> source.
         */
        public void subscribeTemperature() {
            subscribeValue("temperature");
        }
        
        /**
         * Unsubscribes from the <code>temperature</code> source. After a call to this method, the context will no more be notified
         * when a <code>TemperatureSensor</code> device of this composite publishes a value on its <code>temperature</code> source.
         */
        public void unsubscribeTemperature() {
            unsubscribeValue("temperature");
        }
    }
    
    /**
     * A proxy to one <code>TemperatureSensor</code> device
     * 
     * <pre>
     * device TemperatureSensor extends Sensor {
     * 	source temperature as Temperature;
     * }
     * </pre>
     */
    protected final static class TemperatureSensorProxy extends Proxy {
        private TemperatureSensorProxy(Service service, RemoteServiceInfo remoteServiceInfo) {
            super(service, remoteServiceInfo);
        }
        
        /**
         * Subscribes to the <code>temperature</code> source. After a call to this method, the context will be notified when the
         * <code>TemperatureSensor</code> device of this proxy publishes a value on its <code>temperature</code> source.
         */
        public void subscribeTemperature() {
            subscribeValue("temperature");
        }
        
        /**
         * Unsubscribes from the <code>temperature</code> source. After a call to this method, the context will no more be notified
         * when the <code>TemperatureSensor</code> device of this proxy publishes a value on its <code>temperature</code> source.
         */
        public void unsubscribeTemperature() {
            unsubscribeValue("temperature");
        }
        
        /**
         * @return the value of the <code>id</code> attribute
         */
        public java.lang.String id() {
            return (java.lang.String) callGetValue("id");
        }
        
        /**
         * @return the value of the <code>location</code> attribute
         */
        public java.lang.String location() {
            return (java.lang.String) callGetValue("location");
        }
        
        /**
         * @return the value of the <code>user</code> attribute
         */
        public java.lang.String user() {
            return (java.lang.String) callGetValue("user");
        }
    }
    // End of discover part for TemperatureSensor devices
}
