package fr.inria.phoenix.diasuite.framework.controller.message;

import fr.inria.diagen.core.ServiceConfiguration;
import fr.inria.diagen.core.service.composite.CompositeException;
import fr.inria.diagen.core.exception.InvocationException;
import fr.inria.diagen.core.network.RemoteServiceInfo;
import fr.inria.diagen.core.service.local.Service;
import fr.inria.diagen.core.service.proxy.Proxy;

import fr.inria.phoenix.diasuite.framework.context.alert.AlertValue;

/**
 * <pre>
 * controller Message {
 * 	when provided Alert
 * 	do SendMessage on Messenger;
 * }
 * </pre>
 */
@SuppressWarnings("all")
public abstract class AbstractMessage extends Service {
    
    public AbstractMessage(ServiceConfiguration serviceConfiguration) {
        super("/Controller/Message/", serviceConfiguration);
    }
    
    // Methods from the Service class
    @Override
    protected final void internalPostInitialize() {
        subscribeValue("alert", "/Context/Alert/"); // subscribe to Alert context
        postInitialize();
    }
    
    @Override
    public final void valueReceived(java.util.Map<String, Object> properties, RemoteServiceInfo source, String eventName, Object value, Object... indexes) {
        if (eventName.equals("alert") && source.isCompatible("/Context/Alert/")) {
            AlertValue alertValue = new AlertValue((java.lang.String) value);
            
            onAlert(alertValue, new DiscoverForAlert());
        }
    }
    // End of methods from the Service class
    
    // Interaction contract implementation
    /**
     * This method is called when the <code>Alert</code> context publishes a value.
     * 
     * <pre>
     * when provided Alert
     * 	do SendMessage on Messenger;
     * </pre>
     * 
     * @param alert the value of the <code>Alert</code> context.
     * @param discover a discover object to get context values and action methods
     */
    protected abstract void onAlert(AlertValue alert, DiscoverForAlert discover);
    
    // End of interaction contract implementation
    
    // Discover object for Alert
    /**
     * An object to discover devices and contexts for the following interaction contract:
     * 
     * <code>
     * when provided Alert
     * 	do SendMessage on Messenger;
     * </code>
     */
    protected final class DiscoverForAlert {
        private final MessengerDiscovererForAlert messengerDiscoverer = new MessengerDiscovererForAlert(AbstractMessage.this);
        
        /**
         * @return a {@link MessengerDiscovererForAlert} object to discover <code>Messenger</code> devices
         */
        public MessengerDiscovererForAlert messengers() {
            return messengerDiscoverer;
        }
    }
    
    /**
     * Discover object that will exposes the <code>Messenger</code> devices to execute action on for the
     * <code>when provided Alert</code> interaction contract.
     * <p>
     * ------------------
     * MESSENGERS
     * ------------------------------------
     * 
     * <pre>
     * device Messenger extends PhysicalDevice {
     * 	action SendMessage;
     * }
     * </pre>
     */
    protected final static class MessengerDiscovererForAlert {
        private Service serviceParent;
        
        private MessengerDiscovererForAlert(Service serviceParent) {
            super();
            this.serviceParent = serviceParent;
        }
        
        private MessengerCompositeForAlert instantiateComposite() {
            return new MessengerCompositeForAlert(serviceParent);
        }
        
        /**
         * Returns a composite of all accessible <code>Messenger</code> devices
         * 
         * @return a {@link MessengerCompositeForAlert} object composed of all discoverable <code>Messenger</code>
         */
        public MessengerCompositeForAlert all() {
            return instantiateComposite();
        }
        
        /**
         * Returns a proxy to one out of all accessible <code>Messenger</code> devices
         * 
         * @return a {@link MessengerProxyForAlert} object pointing to a random discoverable <code>Messenger</code> device
         */
        public MessengerProxyForAlert anyOne() {
            return all().anyOne();
        }
        
        /**
         * Returns a composite of all accessible <code>Messenger</code> devices whose attribute <code>id</code> matches a given value.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return a {@link MessengerCompositeForAlert} object composed of all matching <code>Messenger</code> devices
         */
        public MessengerCompositeForAlert whereId(java.lang.String id) throws CompositeException {
            return instantiateComposite().andId(id);
        }
        
        /**
         * Returns a composite of all accessible <code>Messenger</code> devices whose attribute <code>location</code> matches a given value.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return a {@link MessengerCompositeForAlert} object composed of all matching <code>Messenger</code> devices
         */
        public MessengerCompositeForAlert whereLocation(java.lang.String location) throws CompositeException {
            return instantiateComposite().andLocation(location);
        }
        
        /**
         * Returns a composite of all accessible <code>Messenger</code> devices whose attribute <code>user</code> matches a given value.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return a {@link MessengerCompositeForAlert} object composed of all matching <code>Messenger</code> devices
         */
        public MessengerCompositeForAlert whereUser(java.lang.String user) throws CompositeException {
            return instantiateComposite().andUser(user);
        }
    }
    
    /**
     * A composite of several <code>Messenger</code> devices to execute action on for the
     * <code>when provided Alert</code> interaction contract.
     * <p>
     * ------------------
     * MESSENGERS
     * ------------------------------------
     * 
     * <pre>
     * device Messenger extends PhysicalDevice {
     * 	action SendMessage;
     * }
     * </pre>
     */
    protected final static class MessengerCompositeForAlert extends fr.inria.diagen.core.service.composite.Composite<MessengerProxyForAlert> {
        private MessengerCompositeForAlert(Service serviceParent) {
            super(serviceParent, "/Device/Device/PhysicalDevice/Messenger/");
        }
        
        @Override
        protected MessengerProxyForAlert instantiateProxy(RemoteServiceInfo rsi) {
            return new MessengerProxyForAlert(service, rsi);
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>id</code>.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return this {@link MessengerCompositeForAlert}, filtered using the attribute <code>id</code>.
         */
        public MessengerCompositeForAlert andId(java.lang.String id) throws CompositeException {
            filterByAttribute("id", id);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>location</code>.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return this {@link MessengerCompositeForAlert}, filtered using the attribute <code>location</code>.
         */
        public MessengerCompositeForAlert andLocation(java.lang.String location) throws CompositeException {
            filterByAttribute("location", location);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>user</code>.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return this {@link MessengerCompositeForAlert}, filtered using the attribute <code>user</code>.
         */
        public MessengerCompositeForAlert andUser(java.lang.String user) throws CompositeException {
            filterByAttribute("user", user);
            return this;
        }
        
        /**
         * Executes the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> action's method on all devices of this composite.
         * 
         * @param to the <code>to</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         * @param title the <code>title</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         * @param content the <code>content</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         * @param attachments the <code>attachments</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         */
        public void sendMessage(fr.inria.phoenix.diasuite.framework.datatype.contact.Contact to,
                java.lang.String title,
                java.lang.String content,
                java.util.List<fr.inria.phoenix.diasuite.framework.datatype.file.File> attachments) throws InvocationException {
            launchDiscovering();
            for (MessengerProxyForAlert proxy : proxies) {
                proxy.sendMessage(to, title, content, attachments);
            }
        }
        
        /**
         * Executes the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> action's method on all devices of this composite.
         * 
         * @param to the <code>to</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         * @param title the <code>title</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         * @param content the <code>content</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         * @param image the <code>image</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         */
        public void sendMessageWithImage(fr.inria.phoenix.diasuite.framework.datatype.contact.Contact to,
                java.lang.String title,
                java.lang.String content,
                java.lang.String image) throws InvocationException {
            launchDiscovering();
            for (MessengerProxyForAlert proxy : proxies) {
                proxy.sendMessageWithImage(to, title, content, image);
            }
        }
    }
    
    /**
     * A proxy to one <code>Messenger</code> device to execute action on for the
     * <code>when provided Alert</code> interaction contract.
     * <p>
     * ------------------
     * MESSENGERS
     * ------------------------------------
     * 
     * <pre>
     * device Messenger extends PhysicalDevice {
     * 	action SendMessage;
     * }
     * </pre>
     */
    protected final static class MessengerProxyForAlert extends Proxy {
        private MessengerProxyForAlert(Service service, RemoteServiceInfo remoteServiceInfo) {
            super(service, remoteServiceInfo);
        }
        
        /**
         * Executes the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> action's method on this device.
         * 
         * @param to the <code>to</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         * @param title the <code>title</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         * @param content the <code>content</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         * @param attachments the <code>attachments</code> parameter of the <code>sendMessage(to as Contact, title as String, content as String, attachments as File [])</code> method.
         */
        public void sendMessage(fr.inria.phoenix.diasuite.framework.datatype.contact.Contact to,
                java.lang.String title,
                java.lang.String content,
                java.util.List<fr.inria.phoenix.diasuite.framework.datatype.file.File> attachments) throws InvocationException {
            callOrder("sendMessage", to, title, content, attachments);
        }
        
        /**
         * Executes the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> action's method on this device.
         * 
         * @param to the <code>to</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         * @param title the <code>title</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         * @param content the <code>content</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         * @param image the <code>image</code> parameter of the <code>sendMessageWithImage(to as Contact, title as String, content as String, image as String)</code> method.
         */
        public void sendMessageWithImage(fr.inria.phoenix.diasuite.framework.datatype.contact.Contact to,
                java.lang.String title,
                java.lang.String content,
                java.lang.String image) throws InvocationException {
            callOrder("sendMessageWithImage", to, title, content, image);
        }
        
        /**
         * @return the value of the <code>id</code> attribute
         */
        public java.lang.String id() {
            return (java.lang.String) callGetValue("id");
        }
        
        /**
         * @return the value of the <code>location</code> attribute
         */
        public java.lang.String location() {
            return (java.lang.String) callGetValue("location");
        }
        
        /**
         * @return the value of the <code>user</code> attribute
         */
        public java.lang.String user() {
            return (java.lang.String) callGetValue("user");
        }
    }
    // End of discover object for Alert
}
