package fr.inria.phoenix.diasuite.framework.controller.timercontroller;

import fr.inria.diagen.core.ServiceConfiguration;
import fr.inria.diagen.core.service.composite.CompositeException;
import fr.inria.diagen.core.exception.InvocationException;
import fr.inria.diagen.core.network.RemoteServiceInfo;
import fr.inria.diagen.core.service.local.Service;
import fr.inria.diagen.core.service.proxy.Proxy;

import fr.inria.phoenix.diasuite.framework.context.event.EventValue;

/**
 * <pre>
 * controller TimerController {
 * 	when provided Event
 * 	do ScheduleTimer on Timer;
 * }
 * </pre>
 */
@SuppressWarnings("all")
public abstract class AbstractTimerController extends Service {
    
    public AbstractTimerController(ServiceConfiguration serviceConfiguration) {
        super("/Controller/TimerController/", serviceConfiguration);
    }
    
    // Methods from the Service class
    @Override
    protected final void internalPostInitialize() {
        subscribeValue("event", "/Context/Event/"); // subscribe to Event context
        postInitialize();
    }
    
    @Override
    public final void valueReceived(java.util.Map<String, Object> properties, RemoteServiceInfo source, String eventName, Object value, Object... indexes) {
        if (eventName.equals("event") && source.isCompatible("/Context/Event/")) {
            EventValue eventValue = new EventValue((java.lang.String) value,
                    (java.lang.Integer) indexes[0]);
            
            onEvent(eventValue, new DiscoverForEvent());
        }
    }
    // End of methods from the Service class
    
    // Interaction contract implementation
    /**
     * This method is called when the <code>Event</code> context publishes a value.
     * 
     * <pre>
     * when provided Event
     * 	do ScheduleTimer on Timer;
     * </pre>
     * 
     * @param event the value of the <code>Event</code> context.
     * @param discover a discover object to get context values and action methods
     */
    protected abstract void onEvent(EventValue event, DiscoverForEvent discover);
    
    // End of interaction contract implementation
    
    // Discover object for Event
    /**
     * An object to discover devices and contexts for the following interaction contract:
     * 
     * <code>
     * when provided Event
     * 	do ScheduleTimer on Timer;
     * </code>
     */
    protected final class DiscoverForEvent {
        private final TimerDiscovererForEvent timerDiscoverer = new TimerDiscovererForEvent(AbstractTimerController.this);
        
        /**
         * @return a {@link TimerDiscovererForEvent} object to discover <code>Timer</code> devices
         */
        public TimerDiscovererForEvent timers() {
            return timerDiscoverer;
        }
    }
    
    /**
     * Discover object that will exposes the <code>Timer</code> devices to execute action on for the
     * <code>when provided Event</code> interaction contract.
     * 
     * <pre>
     * device Timer extends BuiltIn {
     * 	source timerTriggered as String indexed by timerId as String;
     * 	action ScheduleTimer;
     * }
     * </pre>
     */
    protected final static class TimerDiscovererForEvent {
        private Service serviceParent;
        
        private TimerDiscovererForEvent(Service serviceParent) {
            super();
            this.serviceParent = serviceParent;
        }
        
        private TimerCompositeForEvent instantiateComposite() {
            return new TimerCompositeForEvent(serviceParent);
        }
        
        /**
         * Returns a composite of all accessible <code>Timer</code> devices
         * 
         * @return a {@link TimerCompositeForEvent} object composed of all discoverable <code>Timer</code>
         */
        public TimerCompositeForEvent all() {
            return instantiateComposite();
        }
        
        /**
         * Returns a proxy to one out of all accessible <code>Timer</code> devices
         * 
         * @return a {@link TimerProxyForEvent} object pointing to a random discoverable <code>Timer</code> device
         */
        public TimerProxyForEvent anyOne() {
            return all().anyOne();
        }
        
        /**
         * Returns a composite of all accessible <code>Timer</code> devices whose attribute <code>id</code> matches a given value.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return a {@link TimerCompositeForEvent} object composed of all matching <code>Timer</code> devices
         */
        public TimerCompositeForEvent whereId(java.lang.String id) throws CompositeException {
            return instantiateComposite().andId(id);
        }
    }
    
    /**
     * A composite of several <code>Timer</code> devices to execute action on for the
     * <code>when provided Event</code> interaction contract.
     * 
     * <pre>
     * device Timer extends BuiltIn {
     * 	source timerTriggered as String indexed by timerId as String;
     * 	action ScheduleTimer;
     * }
     * </pre>
     */
    protected final static class TimerCompositeForEvent extends fr.inria.diagen.core.service.composite.Composite<TimerProxyForEvent> {
        private TimerCompositeForEvent(Service serviceParent) {
            super(serviceParent, "/Device/Device/Service/BuiltIn/Timer/");
        }
        
        @Override
        protected TimerProxyForEvent instantiateProxy(RemoteServiceInfo rsi) {
            return new TimerProxyForEvent(service, rsi);
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>id</code>.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return this {@link TimerCompositeForEvent}, filtered using the attribute <code>id</code>.
         */
        public TimerCompositeForEvent andId(java.lang.String id) throws CompositeException {
            filterByAttribute("id", id);
            return this;
        }
        
        /**
         * Executes the <code>schedule(id as String, delayMs as Integer)</code> action's method on all devices of this composite.
         * 
         * @param id the <code>id</code> parameter of the <code>schedule(id as String, delayMs as Integer)</code> method.
         * @param delayMs the <code>delayMs</code> parameter of the <code>schedule(id as String, delayMs as Integer)</code> method.
         */
        public void schedule(java.lang.String id,
                java.lang.Integer delayMs) throws InvocationException {
            launchDiscovering();
            for (TimerProxyForEvent proxy : proxies) {
                proxy.schedule(id, delayMs);
            }
        }
        
        /**
         * Executes the <code>periodicSchedule(id as String, delayMs as Integer, periodMs as Integer)</code> action's method on all devices of this composite.
         * 
         * @param id the <code>id</code> parameter of the <code>periodicSchedule(id as String, delayMs as Integer, periodMs as Integer)</code> method.
         * @param delayMs the <code>delayMs</code> parameter of the <code>periodicSchedule(id as String, delayMs as Integer, periodMs as Integer)</code> method.
         * @param periodMs the <code>periodMs</code> parameter of the <code>periodicSchedule(id as String, delayMs as Integer, periodMs as Integer)</code> method.
         */
        public void periodicSchedule(java.lang.String id,
                java.lang.Integer delayMs,
                java.lang.Integer periodMs) throws InvocationException {
            launchDiscovering();
            for (TimerProxyForEvent proxy : proxies) {
                proxy.periodicSchedule(id, delayMs, periodMs);
            }
        }
        
        /**
         * Executes the <code>cancel(id as String)</code> action's method on all devices of this composite.
         * 
         * @param id the <code>id</code> parameter of the <code>cancel(id as String)</code> method.
         */
        public void cancel(java.lang.String id) throws InvocationException {
            launchDiscovering();
            for (TimerProxyForEvent proxy : proxies) {
                proxy.cancel(id);
            }
        }
    }
    
    /**
     * A proxy to one <code>Timer</code> device to execute action on for the
     * <code>when provided Event</code> interaction contract.
     * 
     * <pre>
     * device Timer extends BuiltIn {
     * 	source timerTriggered as String indexed by timerId as String;
     * 	action ScheduleTimer;
     * }
     * </pre>
     */
    protected final static class TimerProxyForEvent extends Proxy {
        private TimerProxyForEvent(Service service, RemoteServiceInfo remoteServiceInfo) {
            super(service, remoteServiceInfo);
        }
        
        /**
         * Executes the <code>schedule(id as String, delayMs as Integer)</code> action's method on this device.
         * 
         * @param id the <code>id</code> parameter of the <code>schedule(id as String, delayMs as Integer)</code> method.
         * @param delayMs the <code>delayMs</code> parameter of the <code>schedule(id as String, delayMs as Integer)</code> method.
         */
        public void schedule(java.lang.String id,
                java.lang.Integer delayMs) throws InvocationException {
            callOrder("schedule", id, delayMs);
        }
        
        /**
         * Executes the <code>periodicSchedule(id as String, delayMs as Integer, periodMs as Integer)</code> action's method on this device.
         * 
         * @param id the <code>id</code> parameter of the <code>periodicSchedule(id as String, delayMs as Integer, periodMs as Integer)</code> method.
         * @param delayMs the <code>delayMs</code> parameter of the <code>periodicSchedule(id as String, delayMs as Integer, periodMs as Integer)</code> method.
         * @param periodMs the <code>periodMs</code> parameter of the <code>periodicSchedule(id as String, delayMs as Integer, periodMs as Integer)</code> method.
         */
        public void periodicSchedule(java.lang.String id,
                java.lang.Integer delayMs,
                java.lang.Integer periodMs) throws InvocationException {
            callOrder("periodicSchedule", id, delayMs, periodMs);
        }
        
        /**
         * Executes the <code>cancel(id as String)</code> action's method on this device.
         * 
         * @param id the <code>id</code> parameter of the <code>cancel(id as String)</code> method.
         */
        public void cancel(java.lang.String id) throws InvocationException {
            callOrder("cancel", id);
        }
        
        /**
         * @return the value of the <code>id</code> attribute
         */
        public java.lang.String id() {
            return (java.lang.String) callGetValue("id");
        }
    }
    // End of discover object for Event
}
