package fr.inria.phoenix.diasuite.framework.misc;

import fr.inria.diagen.core.deploy.AbstractDeploy;

import fr.inria.phoenix.diasuite.framework.context.alert.AbstractAlert;
import fr.inria.phoenix.diasuite.framework.context.event.AbstractEvent;
import fr.inria.phoenix.diasuite.framework.context.position.AbstractPosition;
import fr.inria.phoenix.diasuite.framework.context.state.AbstractState;
import fr.inria.phoenix.diasuite.framework.context.temperature.AbstractTemperature;

import fr.inria.phoenix.diasuite.framework.controller.insert.AbstractInsert;
import fr.inria.phoenix.diasuite.framework.controller.message.AbstractMessage;
import fr.inria.phoenix.diasuite.framework.controller.timercontroller.AbstractTimerController;

/**
 * This class should be implemented to bind the implementation of the various components
 */
public abstract class AppComponentBinder extends AbstractDeploy {

    // Context instances
    private AbstractAlert alertInstance = null;
    private AbstractEvent eventInstance = null;
    private AbstractPosition positionInstance = null;
    private AbstractState stateInstance = null;
    private AbstractTemperature temperatureInstance = null;

    // Controller instances
    private AbstractInsert insertInstance = null;
    private AbstractMessage messageInstance = null;
    private AbstractTimerController timerControllerInstance = null;
    
    @Override
    public void deployAll() {
        // Initialization of contexts
        if (alertInstance == null)
            alertInstance = getInstance(getAlertClass());
        if (eventInstance == null)
            eventInstance = getInstance(getEventClass());
        if (positionInstance == null)
            positionInstance = getInstance(getPositionClass());
        if (stateInstance == null)
            stateInstance = getInstance(getStateClass());
        if (temperatureInstance == null)
            temperatureInstance = getInstance(getTemperatureClass());
        // Intialization of controllers
        if (insertInstance == null)
            insertInstance = getInstance(getInsertClass());
        if (messageInstance == null)
            messageInstance = getInstance(getMessageClass());
        if (timerControllerInstance == null)
            timerControllerInstance = getInstance(getTimerControllerClass());
        // Deploying contexts
        deploy(alertInstance);
        deploy(eventInstance);
        deploy(positionInstance);
        deploy(stateInstance);
        deploy(temperatureInstance);
        // Deploying controllers
        deploy(insertInstance);
        deploy(messageInstance);
        deploy(timerControllerInstance);
    }
    
    @Override
    public void undeployAll() {
        // Undeploying contexts
        undeploy(alertInstance);
        undeploy(eventInstance);
        undeploy(positionInstance);
        undeploy(stateInstance);
        undeploy(temperatureInstance);
        // Undeploying controllers
        undeploy(insertInstance);
        undeploy(messageInstance);
        undeploy(timerControllerInstance);
    }
    
    // Abstract binding methods for contexts
    /**
     * Overrides this method to provide the implementation class of the <code>Alert</code> context
     * 
     * <pre>
     * context Alert as String {
     * 	when provided timerTriggered from Timer 
     * 	maybe publish;
     * 	when provided state from PalletTracker
     * 	no publish;
     * }
     * </pre>
     * @return a class object of a derivation of {@link AbstractAlert} that implements the <code>Alert</code> context
     */
    public abstract Class<? extends AbstractAlert> getAlertClass();
    
    /**
     * Overrides this method to provide the implementation class of the <code>Event</code> context
     * 
     * <pre>
     * context Event as String indexed by periodicity as Integer {
     * 	when provided message from Input
     * 	maybe publish;	
     * }
     * </pre>
     * @return a class object of a derivation of {@link AbstractEvent} that implements the <code>Event</code> context
     */
    public abstract Class<? extends AbstractEvent> getEventClass();
    
    /**
     * Overrides this method to provide the implementation class of the <code>Position</code> context
     * <p>
     * * PalletTracker contexts
     * The provided position is per station and the computed one is per device.
     * 
     * <pre>
     * context Position as GPSCoordinates indexed by id as String {
     * 	when provided position from PalletTracker
     * 	no publish;
     * 	when required;
     * }
     * </pre>
     * @return a class object of a derivation of {@link AbstractPosition} that implements the <code>Position</code> context
     */
    public abstract Class<? extends AbstractPosition> getPositionClass();
    
    /**
     * Overrides this method to provide the implementation class of the <code>State</code> context
     * <p>
     * context Warehouse as String indexed by position as GPSCoordinates {
     * when required get warehouse from PalletTrackingDB;
     * }
     * 
     * <pre>
     * context State as PalletState {
     * 	when provided state from PalletTracker 
     * 	get Position, Temperature	 
     * 	always publish;
     * }
     * </pre>
     * @return a class object of a derivation of {@link AbstractState} that implements the <code>State</code> context
     */
    public abstract Class<? extends AbstractState> getStateClass();
    
    /**
     * Overrides this method to provide the implementation class of the <code>Temperature</code> context
     * <p>
     * The provided temperature is indexed by id and published.
     * 
     * <pre>
     * context Temperature as Integer indexed by id as String {
     * 	when provided temperature from TemperatureSensor 
     * 	no publish;
     * 	when required;
     * }
     * </pre>
     * @return a class object of a derivation of {@link AbstractTemperature} that implements the <code>Temperature</code> context
     */
    public abstract Class<? extends AbstractTemperature> getTemperatureClass();
    
    // End of abstract binding methods for contexts
    
    // Abstract binding methods for controllers
    /**
     * Overrides this method to provide the implementation class of the <code>Insert</code> controller
     * <p>
     * * PalletTracker controllers
     * 
     * <pre>
     * controller Insert {
     * 	when provided State
     * 	do InsertPalletState on PalletTrackingDB;
     * }
     * </pre>
     * @return a class object of a derivation of {@link AbstractInsert} that implements the <code>Insert</code> controller
     */
    public abstract Class<? extends AbstractInsert> getInsertClass();
    
    /**
     * Overrides this method to provide the implementation class of the <code>Message</code> controller
     * 
     * <pre>
     * controller Message {
     * 	when provided Alert
     * 	do SendMessage on Messenger;
     * }
     * </pre>
     * @return a class object of a derivation of {@link AbstractMessage} that implements the <code>Message</code> controller
     */
    public abstract Class<? extends AbstractMessage> getMessageClass();
    
    /**
     * Overrides this method to provide the implementation class of the <code>TimerController</code> controller
     * 
     * <pre>
     * controller TimerController {
     * 	when provided Event
     * 	do ScheduleTimer on Timer;
     * }
     * </pre>
     * @return a class object of a derivation of {@link AbstractTimerController} that implements the <code>TimerController</code> controller
     */
    public abstract Class<? extends AbstractTimerController> getTimerControllerClass();
    
    // End of abstract binding methods for controllers
}
