package fr.inria.phoenix.scenario.pallettracker.impl.context;

import java.math.BigDecimal;
import java.sql.Timestamp;
import java.text.DecimalFormat;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import fr.inria.diagen.core.ServiceConfiguration;
import fr.inria.phoenix.diasuite.framework.context.position.AbstractPosition;
import fr.inria.phoenix.diasuite.framework.datatype.gpscoordinates.GPSCoordinates;
import fr.inria.phoenix.diasuite.framework.device.pallettracker.PositionFromPalletTracker;

public class Position extends AbstractPosition {

	private static byte ONE_MINUTE_IN_SECONDS = 60;
	private long lastMessageTimestamp = 0;
	private Map<String, Set<String>> deviceToStationsMap;
	private Map<String, GPSCoordinates> stationGPSMap;
	private Map<String, Float> stationRssiMap;

	public Position(ServiceConfiguration serviceConfiguration) {
		super(serviceConfiguration);

		deviceToStationsMap = Collections
				.synchronizedMap(new HashMap<String, Set<String>>());

		stationGPSMap = Collections
				.synchronizedMap(new HashMap<String, GPSCoordinates>());

		stationRssiMap = Collections
				.synchronizedMap(new HashMap<String, Float>());
	}

	@Override
	protected void onPositionFromPalletTracker(
			PositionFromPalletTracker positionFromPalletTracker) {

		// Message information
		String deviceId = positionFromPalletTracker.sender().id();
		long messageTimeStamp = Long.parseLong(positionFromPalletTracker
				.indices().params().get(0));
		Float rssi = Float.parseFloat(positionFromPalletTracker.indices()
				.params().get(1));

		// Station information
		String stationId = positionFromPalletTracker.indices().params().get(2);
		GPSCoordinates stationPosition = positionFromPalletTracker.value();

		if (lastMessageTimestamp == 0)
			lastMessageTimestamp = messageTimeStamp;

		Timestamp newMessageTimestamp = new Timestamp(messageTimeStamp * 1000);
		Timestamp lastMessageTimestampPlusOneMin = new Timestamp(
				(lastMessageTimestamp + ONE_MINUTE_IN_SECONDS) * 1000);

		if (newMessageTimestamp.after(lastMessageTimestampPlusOneMin)) {
			deviceToStationsMap.clear();
			stationGPSMap.clear();
			stationRssiMap.clear();
		}

		if (deviceToStationsMap.get(deviceId) == null)
			deviceToStationsMap.put(deviceId, new HashSet<String>());

		deviceToStationsMap.get(deviceId).add(stationId);
		stationGPSMap.put(stationId, stationPosition);
		stationRssiMap.put(stationId, rssi);
	}

	@Override
	protected GPSCoordinates getValue(String id) {
		Set<String> stations = deviceToStationsMap.get(id);
		Map<String, Float> signalStrengthRatio = new HashMap<String, Float>();
		Float rssiSum = 0f;
		Float latitude = 0f;
		Float longitude = 0f;

		if (stations != null) {

			for (String station : stations)
				rssiSum += stationRssiMap.get(station);

			for (String station : stations)
				signalStrengthRatio.put(station, stationRssiMap.get(station)
						/ rssiSum);

			for (String station : stations) {
				latitude += stationGPSMap.get(station).getLatitude()
						* signalStrengthRatio.get(station);
			}

			for (String station : stations) {
				longitude += stationGPSMap.get(station).getLongitude()
						* signalStrengthRatio.get(station);
			}

			DecimalFormat df = new DecimalFormat();
			df.setMaximumFractionDigits(2);

			return new GPSCoordinates(round(latitude, 1), round(longitude, 1));
		}

		return null;
	}

	public static float round(float d, int decimalPlace) {
		BigDecimal bd = new BigDecimal(Float.toString(d));
		bd = bd.setScale(decimalPlace, BigDecimal.ROUND_HALF_UP);
		return bd.floatValue();
	}
}
